//+------------------------------------------------------------------+
//|   BB_RSI_RR_EA.mq5   (v1.00 – Low/High Touch, moderate Defaults) |
//+------------------------------------------------------------------+
#property copyright "BND"
#property strict
#include <Trade/Trade.mqh>

// ======  INPUTs  ===================================================
input double   InpRiskPercent   = 1.0;     // Kontorisiko pro Trade (%)
input double   InpRR            = 1.5;     // TP = RR * SL   (ATR‑Multiplikator)
input int      InpRSIPeriod     = 14;
input int      InpRSI_OB        = 65;      // Overbought‑Schwelle
input int      InpRSI_OS        = 35;      // Oversold‑Schwelle
input int      InpBBPeriod      = 20;      // Bollinger‑Periode
input double   InpBBDev         = 1.5;     // Bollinger‑Standardabweichung
input int      InpEMATrend      = 200;     // Haupt‑Trendfilter
input bool     InpUseEMA50Conf  = true;    // zusätzliche EMA‑50‑Bestätigung?
input int      InpATRPeriod     = 10;      // ATR für SL/TP
input ENUM_TIMEFRAMES InpTF     = PERIOD_CURRENT;
input bool     InpUseNewsFilter = false;   // News‑Filter (aus)
input int      InpStartHour     = 0;       // Handelsfenster (Brokerzeit)
input int      InpEndHour       = 24;
input ulong    InpMagic         = 484105;
input bool     InpVerbose       = false;   // Journalausgabe pro Kerze

// ======  GLOBALS  ==================================================
CTrade  trade;
int hRSI, hBB, hEMA200, hEMA50, hATR;

//--------------------------------------------------------------------
int OnInit()
{
   hRSI    = iRSI   (_Symbol,InpTF,InpRSIPeriod,PRICE_CLOSE);
   hBB     = iBands (_Symbol,InpTF,InpBBPeriod,0,InpBBDev,PRICE_CLOSE);
   hEMA200 = iMA    (_Symbol,InpTF,InpEMATrend,0,MODE_EMA,PRICE_CLOSE);
   hATR    = iATR   (_Symbol,InpTF,InpATRPeriod);
   hEMA50  = iMA    (_Symbol,InpTF,50,0,MODE_EMA,PRICE_CLOSE);

   if(hRSI==INVALID_HANDLE || hBB==INVALID_HANDLE ||
      hEMA200==INVALID_HANDLE || hATR==INVALID_HANDLE)
      return(INIT_FAILED);

   Print(__FILE__," v1.05 initialised");
   return(INIT_SUCCEEDED);
}

//--------------------------------------------------------------------
void OnTick()
{
   static datetime lastBar=0;
   datetime bar=iTime(_Symbol,InpTF,0);
   if(bar==lastBar) return;                   // nur einmal pro neuer Kerze
   lastBar=bar;

   if(!IsInsideTradeWindow())            return;
   if(InpUseNewsFilter && IsHighNews())  return;

   // ------- Werte der **abgeschlossenen** Kerze (Shift 1) ----------
   double rsi[1], bbLow[1], bbUp[1], ema200[1], ema50[1], atr[1];
   if(CopyBuffer(hRSI   ,0,1,1,rsi   )<=0) return;
   if(CopyBuffer(hBB    ,2,1,1,bbLow )<=0) return;        // Lower‑Band
   if(CopyBuffer(hBB    ,0,1,1,bbUp  )<=0) return;        // Upper‑Band
   if(CopyBuffer(hEMA200,0,1,1,ema200)<=0) return;
   if(CopyBuffer(hATR   ,0,1,1,atr   )<=0) return;
   if(InpUseEMA50Conf &&
      CopyBuffer(hEMA50,0,1,1,ema50 )<=0) return;

   // Candle‑Daten (Shift 1)
   double close = iClose(_Symbol,InpTF,1);
   double low   = iLow  (_Symbol,InpTF,1);
   double high  = iHigh (_Symbol,InpTF,1);

   // Trendfilter ----------------------------------------------------
   bool upTrend   = close > ema200[0];
   bool downTrend = close < ema200[0];

   if(InpUseEMA50Conf){
      if(upTrend   && close<ema50[0]) upTrend=false;
      if(downTrend && close>ema50[0]) downTrend=false;
   }

   // Signal‑Logik (Low/High‑Touch) ----------------------------------
   bool buySig  = (rsi[0] < InpRSI_OS) && (low  <= bbLow[0]) && upTrend;
   bool sellSig = (rsi[0] > InpRSI_OB) && (high >= bbUp [0]) && downTrend;

   if(InpVerbose)
      PrintFormat("%s  buy:%d sell:%d  C:%.5f  L:%.5f  H:%.5f  BB_L:%.5f  BB_U:%.5f  RSI:%.2f  up:%d dn:%d",
                  TimeToString(bar,TIME_DATE|TIME_MINUTES),buySig,sellSig,
                  close,low,high,bbLow[0],bbUp[0],rsi[0],upTrend,downTrend);

   ManagePositions(buySig,sellSig,atr[0]);
}

//--------------------------------------------------------------------
void ManagePositions(bool buy,bool sell,double atr)
{
   ulong ticket=0; int type=-1; bool have=false;

   for(int i=0;i<PositionsTotal();i++)
   {
      ulong tk=PositionGetTicket(i);
      if(PositionSelectByTicket(tk) &&
         PositionGetInteger(POSITION_MAGIC)==(long)InpMagic &&
         PositionGetString (POSITION_SYMBOL)==_Symbol)
      { ticket=tk; type=(int)PositionGetInteger(POSITION_TYPE); have=true; break; }
   }

   double slPts = atr;             // SL = 1*ATR
   double tpPts = atr * InpRR;     // TP = RR*SL

   if(!have)                       // -------- Neuer Trade ----------
   {
      double lot = LotsFromRisk(slPts);
      if(buy)  Open(ORDER_TYPE_BUY ,lot,slPts,tpPts);
      if(sell) Open(ORDER_TYPE_SELL,lot,slPts,tpPts);
   }
   else                            // -------- Exit bei Gegensignal --
   {
      if((type==POSITION_TYPE_BUY  && sell) ||
         (type==POSITION_TYPE_SELL && buy ))
         trade.PositionClose(ticket);
   }
}

//--------------------------------------------------------------------
void Open(ENUM_ORDER_TYPE t,double lot,double slPts,double tpPts)
{
   double ask   = SymbolInfoDouble(_Symbol,SYMBOL_ASK);
   double bid   = SymbolInfoDouble(_Symbol,SYMBOL_BID);
   double price = (t==ORDER_TYPE_BUY) ? ask : bid;
   double sl    = (t==ORDER_TYPE_BUY) ? price-slPts : price+slPts;
   double tp    = (t==ORDER_TYPE_BUY) ? price+tpPts : price-tpPts;

   trade.SetExpertMagicNumber(InpMagic);
   trade.SetDeviationInPoints(25);
   if(!trade.PositionOpen(_Symbol,t,lot,price,sl,tp))
      Print("Open‑error ",trade.ResultRetcode()," ",trade.ResultRetcodeDescription());
}

//--------------------------------------------------------------------
double LotsFromRisk(double slPts)
{
   double tv   = SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_VALUE);
   double ts   = SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_SIZE);
   double risk = AccountInfoDouble(ACCOUNT_BALANCE)*InpRiskPercent/100.0;
   double step = SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_STEP);
   double min  = SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MIN);

   double contract = (slPts/ts)*tv;
   if(contract<=0.0) return min;

   double lot = NormalizeDouble(risk/contract,(int)MathLog10(1.0/step));
   return MathMax(min,lot);
}

//--------------------------------------------------------------------
bool IsInsideTradeWindow()
{
   MqlDateTime dt; TimeToStruct(TimeCurrent(),dt);
   return(dt.hour >= InpStartHour && dt.hour < InpEndHour);
}

//--------------------------------------------------------------------
// Dummy‑Newsfilter (optional reaktivieren)
bool IsHighNews(){ return false; }
//+------------------------------------------------------------------+
/*
bool IsHighNews()
{
#ifndef __MQL5_ECONOMIC_CALENDAR__
   return(false);
#else
   MqlCalendarValue data[];
   datetime from=TimeCurrent()-InpNewsMinBefore*60,
            to  =TimeCurrent()+InpNewsMinAfter *60;
   return(CalendarValueHistory(data,from,to,CALENDAR_EVENT_HIGH)>0);
#endif
}
*/
//+------------------------------------------------------------------+
